@@ -119,34 +119,16 @@ describe Rack::Utils do
   end
 
   it "not create infinite loops with cycle structures" do
-    ex = { "foo" => nil }
-    ex["foo"] = ex
-
     params = Rack::Utils::KeySpaceConstrainedParams.new
     params['foo'] = params
-    params.to_params_hash.to_s.must_equal ex.to_s
+    h = params.to_params_hash
+    h['foo'].to_s.must_equal h['foo']['foo'].to_s
   end
 
   it "parse nil as an empty query string" do
     Rack::Utils.parse_nested_query(nil).must_equal({})
   end
 
-  deprecated "should warn for deprecated QueryParser.make_default call with key_space_limit" do
-    Rack::QueryParser.make_default(1, 1).must_be_kind_of Rack::QueryParser
-  end
-
-  deprecated "should warn using deprecated QueryParser.new call with key_space_limit" do
-    Rack::QueryParser.new(Rack::QueryParser::Params, 1, 1).must_be_kind_of Rack::QueryParser
-  end
-
-  deprecated "should warn using deprecated Rack::Util.key_space_limit=" do
-    Rack::Utils.key_space_limit = 65536
-  end
-
-  deprecated "should warn using deprecated Rack::Util.key_space_limit" do
-    Rack::Utils.key_space_limit
-  end
-
   it "raise an exception if the params are too deep" do
     len = Rack::Utils.param_depth_limit
 
@@ -256,9 +238,10 @@ describe Rack::Utils do
     lambda { Rack::Utils.parse_nested_query("x[y]=1&x[y][][w]=2") }.
       must_raise(Rack::Utils::ParameterTypeError).
       message.must_equal "expected Array (got String) for param `y'"
+  end
 
-    Rack::Utils.parse_nested_query("foo%81E=1").
-      must_equal "foo\x81E"=>"1"
+  it "can parse a query string with a key that has invalid UTF-8 encoded bytes" do
+    Rack::Utils.parse_nested_query("foo%81E=1").must_equal "foo\x81E"=>"1"
   end
 
   it "only moves to a new array when the full key has been seen" do
@@ -292,8 +275,7 @@ describe Rack::Utils do
       default_parser = Rack::Utils.default_query_parser
       param_parser_class = Class.new(Rack::QueryParser::Params) do
         def initialize(*)
-          super
-          @params = Hash.new{|h, k| h[k.to_s] if k.is_a?(Symbol)}
+          super(){|h, k| h[k.to_s] if k.is_a?(Symbol)}
         end
       end
       Rack::Utils.default_query_parser = Rack::QueryParser.new(param_parser_class, 100)
@@ -493,18 +475,10 @@ describe Rack::Utils do
     Rack::Utils.escape_html("f&o").must_equal "f&amp;o"
     Rack::Utils.escape_html("f<o").must_equal "f&lt;o"
     Rack::Utils.escape_html("f>o").must_equal "f&gt;o"
-    Rack::Utils.escape_html("f'o").must_equal "f&#x27;o"
+    Rack::Utils.escape_html("f'o").must_equal "f&#39;o"
     Rack::Utils.escape_html('f"o').must_equal "f&quot;o"
-    Rack::Utils.escape_html("f/o").must_equal "f&#x2F;o"
-    Rack::Utils.escape_html("<foo></foo>").must_equal "&lt;foo&gt;&lt;&#x2F;foo&gt;"
-  end
-
-  it "escape html entities even on MRI when it's bugged" do
-    test_escape = lambda do
-      Rack::Utils.escape_html("\300<").must_equal "\300&lt;"
-    end
-
-    test_escape.must_raise ArgumentError
+    Rack::Utils.escape_html("<foo></foo>").must_equal "&lt;foo&gt;&lt;/foo&gt;"
+    Rack::Utils.escape_html("\300<").must_equal "\300&lt;"
   end
 
   it "escape html entities in unicode strings" do
@@ -512,6 +486,11 @@ describe Rack::Utils do
     Rack::Utils.escape_html("☃").must_equal "☃"
   end
 
+  it 'escape_html handles non-strings' do
+    Rack::Utils.escape_html(nil).must_equal ""
+    Rack::Utils.escape_html(123).must_equal "123"
+  end
+
   it "figure out which encodings are acceptable" do
     helper = lambda do |a, b|
       Rack::Request.new(Rack::MockRequest.env_for("", "HTTP_ACCEPT_ENCODING" => a))
@@ -554,10 +533,31 @@ describe Rack::Utils do
     Rack::Utils.status_code(:ok).must_equal 200
   end
 
+  it "return status code and give deprecation warning for obsolete symbols" do
+    replaced_statuses = {
+      payload_too_large: {status_code: 413, standard_symbol: :content_too_large},
+      unprocessable_entity: {status_code: 422, standard_symbol: :unprocessable_content}
+    }
+    dropped_statuses = {bandwidth_limit_exceeded: 509, not_extended: 510}
+
+    capture_warnings(Rack::Utils) do |warnings|
+      replaced_statuses.each do |symbol, value_hash|
+        Rack::Utils.status_code(symbol).must_equal value_hash[:status_code]
+        warnings.must_be_empty
+      end
+
+      dropped_statuses.each do |symbol, code|
+        Rack::Utils.status_code(symbol).must_equal code
+        warnings.pop.must_equal ["Status code #{symbol.inspect} is deprecated and will be removed in a future version of Rack.", { uplevel: 3 }]
+      end
+    end
+  end
+
   it "raise an error for an invalid symbol" do
-    assert_raises(ArgumentError, "Unrecognized status code :foobar") do
+    error = assert_raises(ArgumentError) do
       Rack::Utils.status_code(:foobar)
     end
+    error.message.must_equal "Unrecognized status code :foobar"
   end
 
   it "return rfc2822 format from rfc2822 helper" do
@@ -618,34 +618,6 @@ describe Rack::Utils, "cookies" do
     Rack::Utils.set_cookie_header('name', %w[va ue]).must_equal 'name=va&ue'
   end
 
-  deprecated "adds new cookies to nil header" do
-    Rack::Utils.add_cookie_to_header(nil, 'name', 'value').must_equal 'name=value'
-  end
-
-  deprecated "adds new cookies to blank header" do
-    header = ''
-    Rack::Utils.add_cookie_to_header(header, 'name', 'value').must_equal 'name=value'
-    header.must_equal ''
-  end
-
-  deprecated "adds new cookies to string header" do
-    header = 'existing-cookie'
-    Rack::Utils.add_cookie_to_header(header, 'name', 'value').must_equal ["existing-cookie", "name=value"]
-    header.must_equal 'existing-cookie'
-  end
-
-  deprecated "adds new cookies to array header" do
-    header = %w[ existing-cookie ]
-    Rack::Utils.add_cookie_to_header(header, 'name', 'value').must_equal ["existing-cookie", "name=value"]
-    header.must_equal %w[ existing-cookie ]
-  end
-
-  deprecated "adds new cookies to an unrecognized header" do
-    lambda {
-      Rack::Utils.add_cookie_to_header(Object.new, 'name', 'value')
-    }.must_raise ArgumentError
-  end
-
   it "sets and deletes cookies in header hash" do
     headers = {}
     Rack::Utils.set_cookie_header!(headers, 'name', 'value')
@@ -657,13 +629,20 @@ describe Rack::Utils, "cookies" do
   end
 
   it "encodes cookie key values by default" do
-    Rack::Utils.set_cookie_header('na e', 'value').must_equal 'na+e=value'
+    capture_warnings(Rack::Utils) do |warnings|
+      Rack::Utils.set_cookie_header('na e', 'value').must_equal 'na+e=value'
+      warnings.pop.first.must_match(/Cookie key "na e" is not valid/)
+    end
   end
 
   it "does not encode cookie key values if :escape_key is false" do
     Rack::Utils.set_cookie_header('na e', value: 'value', escape_key: false).must_equal 'na e=value'
   end
 
+  it "sets partitioned cookie attribute" do
+    Rack::Utils.set_cookie_header('name', {value: 'value', partitioned: true}).must_equal 'name=value; partitioned'
+  end
+
   it "deletes cookies in header field" do
     header = []
 
@@ -706,13 +685,6 @@ describe Rack::Utils, "cookies" do
     Rack::Utils.delete_cookie_header!(header, 'name').must_be_nil
     header['set-cookie'].must_equal "name=; max-age=0; expires=Thu, 01 Jan 1970 00:00:00 GMT"
   end
-
-  deprecated "sets deleted cookie" do
-    Rack::Utils.make_delete_cookie_header(nil, 'name', {}).
-      must_equal "name=; max-age=0; expires=Thu, 01 Jan 1970 00:00:00 GMT"
-    Rack::Utils.add_remove_cookie_to_header(nil, 'name').
-      must_equal "name=; max-age=0; expires=Thu, 01 Jan 1970 00:00:00 GMT"
-  end
 end
 
 describe Rack::Utils, "get_byte_ranges" do
@@ -720,7 +692,7 @@ describe Rack::Utils, "get_byte_ranges" do
     assert_equal [], Rack::Utils.byte_ranges({ "HTTP_RANGE" => "bytes=0-20,0-500" }, 500)
   end
 
-  deprecated "pase simple byte ranges from env" do
+  it "parse simple byte ranges from env" do
     Rack::Utils.byte_ranges({ "HTTP_RANGE" => "bytes=123-456" }, 500).must_equal [(123..456)]
   end
 
@@ -762,46 +734,11 @@ describe Rack::Utils, "get_byte_ranges" do
   end
 
   it "handle byte ranges of empty files" do
-    Rack::Utils.get_byte_ranges("bytes=123-456", 0).must_equal []
-    Rack::Utils.get_byte_ranges("bytes=0-", 0).must_equal []
-    Rack::Utils.get_byte_ranges("bytes=-100", 0).must_equal []
-    Rack::Utils.get_byte_ranges("bytes=0-0", 0).must_equal []
-    Rack::Utils.get_byte_ranges("bytes=-0", 0).must_equal []
-  end
-end
-
-describe Rack::Utils::HeaderHash do
-  deprecated ".[] returns Rack::Headers as is if not frozen" do
-    h1 = Rack::Headers["Content-MD5" => "d5ff4e2a0 ..."]
-    h2 = Rack::Utils::HeaderHash[h1]
-    h2.must_be_same_as h1
-    h3 = Rack::Utils::HeaderHash[h1.freeze]
-    h3.wont_be_same_as h1
-    h3.must_equal h1
-  end
-
-  deprecated ".[] returns instance of Rack::Headers" do
-    h = Rack::Utils::HeaderHash["Content-MD5" => "d5ff4e2a0 ..."]
-    h.must_be_kind_of Rack::Headers
-    h['content-md5'].must_equal "d5ff4e2a0 ..."
-
-    h = Rack::Utils::HeaderHash[[["Content-MD5","d5ff4e2a0 ..."]]]
-    h.must_be_kind_of Rack::Headers
-    h['content-md5'].must_equal "d5ff4e2a0 ..."
-  end
-
-  deprecated ".new returns instance of Rack::Headers" do
-    h = Rack::Utils::HeaderHash.new("Content-MD5" => "d5ff4e2a0 ...")
-    h.must_be_kind_of Rack::Headers
-    h['content-md5'].must_equal "d5ff4e2a0 ..."
-
-    h = Rack::Utils::HeaderHash.new([["Content-MD5","d5ff4e2a0 ..."]])
-    h.must_be_kind_of Rack::Headers
-    h['content-md5'].must_equal "d5ff4e2a0 ..."
-  end
-
-  it ".allocate raises" do
-    proc { Rack::Utils::HeaderHash.allocate }.must_raise TypeError
+    Rack::Utils.get_byte_ranges("bytes=123-456", 0).must_be_nil
+    Rack::Utils.get_byte_ranges("bytes=0-", 0).must_be_nil
+    Rack::Utils.get_byte_ranges("bytes=-100", 0).must_be_nil
+    Rack::Utils.get_byte_ranges("bytes=0-0", 0).must_be_nil
+    Rack::Utils.get_byte_ranges("bytes=-0", 0).must_be_nil
   end
 end
 
